@@ -26,7 +26,7 @@ pub fn pair() -> Term {
     )))
 }
 
-/// Applied to a Church-encoded pair (a, b) it yields a.
+/// Applied to a Church-encoded pair `(a, b)` it yields `a`.
 ///
 /// FIRST := λp.p TRUE = λ 1 TRUE
 ///
@@ -42,7 +42,7 @@ pub fn pair() -> Term {
 /// ```
 pub fn first() -> Term { abs(Var(1).app(tru())) }
 
-/// Applied to a Church-encoded pair (a, b) it yields b.
+/// Applied to a Church-encoded pair `(a, b)` it yields `b`.
 ///
 /// SECOND := λp.p FALSE = λ 1 FALSE
 ///
@@ -59,7 +59,7 @@ pub fn first() -> Term { abs(Var(1).app(tru())) }
 pub fn second() -> Term { abs(Var(1).app(fls())) }
 
 impl Term {
-    /// Checks whether self is a Church-encoded pair.
+    /// Checks whether `self` is a Church-encoded pair.
     ///
     /// # Example
     /// ```
@@ -74,7 +74,7 @@ impl Term {
         self.fst_ref().is_ok() && self.snd_ref().is_ok()
     }
 
-    /// Splits a Church-encoded pair into a pair of terms, consuming self.
+    /// Splits a Church-encoded pair into a pair of terms, consuming `self`.
     ///
     /// # Example
     /// ```
@@ -155,7 +155,7 @@ impl Term {
         }
     }
 
-    /// Returns the first term from a Church-encoded pair, consuming self.
+    /// Returns the first term from a Church-encoded pair, consuming `self`.
     ///
     /// # Example
     /// ```
@@ -201,7 +201,7 @@ impl Term {
         Ok(try!(self.unpair_ref_mut()).0)
     }
 
-    /// Returns the second term from a Church-encoded pair, consuming self.
+    /// Returns the second term from a Church-encoded pair, consuming `self`.
     ///
     /// # Example
     /// ```
@@ -247,23 +247,36 @@ impl Term {
     }
 }
 
+impl From<(Term, Term)> for Term {
+    fn from((t1, t2): (Term, Term)) -> Self {
+        abs(
+            Var(1)
+            .app(t1)
+            .app(t2)
+        )
+    }
+}
 
 #[cfg(test)]
 mod test {
     use super::*;
-    use arithmetic::*;
-    use reduction::*;
+    use reduction::normalize;
+
+    #[test]
+    fn pair_from_pair() {
+        assert_eq!(Term::from((0.into(), 1.into())), normalize(pair().app(0.into()).app(1.into())));
+    }
 
     #[test]
     fn pair_operations() {
-        let pair_four_three = normalize(pair().app(to_cnum(4)).app(to_cnum(3)));
+        let pair_four_three = normalize(pair().app(4.into()).app(3.into()));
 
         assert!(pair_four_three.is_pair());
 
-        assert_eq!(pair_four_three.fst_ref(), Ok(&to_cnum(4)));
-        assert_eq!(pair_four_three.snd_ref(), Ok(&to_cnum(3)));
+        assert_eq!(pair_four_three.fst_ref(), Ok(&4.into()));
+        assert_eq!(pair_four_three.snd_ref(), Ok(&3.into()));
 
         let unpaired = pair_four_three.unpair();
-        assert_eq!(unpaired, Ok((to_cnum(4), to_cnum(3))));
+        assert_eq!(unpaired, Ok((4.into(), 3.into())));
     }
 }
