@@ -1,4 +1,4 @@
-//! [β-reduction](https://en.wikipedia.org/wiki/Beta_normal_form)
+//! [β-reduction](https://en.wikipedia.org/wiki/Beta_normal_form) for lambda `Term`s
 
 use term::*;
 use term::Term::*;
@@ -20,14 +20,15 @@ enum Expression {
 
 type Environment = VecDeque<Closure>;
 
-fn whnf(term: Term, env: Environment) -> Expression {
+fn whnf(term: Term, mut env: Environment) -> Expression {
     match term {
-        Var(i) => match env[i - 1].clone() {
-            Depth(i) => Evaluation(Depth(i)),
-            TermInEnv(t, e) => whnf(t, e)
+        Var(i) => match env.remove(i - 1) {
+            Some(Depth(i)) => Evaluation(Depth(i)),
+            Some(TermInEnv(t, e)) => whnf(t, e),
+            None => unreachable!()
         },
         Abs(t) => Evaluation(TermInEnv(Abs(t), env)),
-        App(lhs, rhs) => match whnf(*lhs, env.clone()) {
+        App(lhs, rhs) => match whnf(*lhs, env.clone()) { // TODO: try to remove this clone()
             Evaluation(TermInEnv(Abs(lhs2), mut env2)) => {
                 env2.push_front(TermInEnv(*rhs, env));
                 whnf(*lhs2, env2)
@@ -59,12 +60,11 @@ fn nf(depth: usize, term: Term, env: Environment) -> Term {
 ///
 /// ```
 /// use lambda_calculus::booleans::{if_else, tru};
-/// use lambda_calculus::arithmetic::{zero, one};
+/// use lambda_calculus::arithmetic::{zero, one, succ};
 /// use lambda_calculus::reduction::normalize;
 ///
-/// let if_true_then_one_else_zero = if_else().app(tru()).app(one()).app(zero());
-///
-/// assert_eq!(normalize(if_true_then_one_else_zero), one());
+/// assert_eq!(normalize(succ().app(zero())), one());
+/// assert_eq!(normalize(if_else().app(tru()).app(one()).app(zero())), one());
 /// ```
 pub fn normalize(term: Term) -> Term {
     nf(0, term, VecDeque::new())
@@ -72,17 +72,11 @@ pub fn normalize(term: Term) -> Term {
 
 #[cfg(test)]
 mod test {
-    use arithmetic::*;
-    use super::*;
+//    use arithmetic::*;
+//    use super::*;
 
     #[test]
     fn weak_head_normal_form() {
         // TODO
     }
-
-    #[test]
-    fn normal_form() {
-        // TODO: more tests, esp. generic ones
-        assert_eq!(normalize(succ().app(zero())), one());
-    }
 }
