@@ -26,7 +26,7 @@ pub enum Error {
 }
 
 impl Term {
-    /// Applies self to another term.
+    /// Applies `self` to another term without substitution or reduction.
     ///
     /// # Example
     /// ```
@@ -145,7 +145,7 @@ impl Term {
         }
     }
 
-    /// Returns the left-hand side term of an application. Consumes self.
+    /// Returns the left-hand side term of an application. Consumes `self`.
     ///
     /// # Example
     /// ```
@@ -181,7 +181,7 @@ impl Term {
         if let Ok((lhs, _)) = self.unapp_ref_mut() { Ok(lhs) } else { Err(NotAnApp) }
     }
 
-    /// Returns the right-hand side term of an application. Consumes self.
+    /// Returns the right-hand side term of an application. Consumes `self`.
     ///
     /// # Example
     /// ```
@@ -216,6 +216,45 @@ impl Term {
     pub fn rhs_ref_mut(&mut self) -> Result<&mut Term, Error> {
         if let Ok((_, rhs)) = self.unapp_ref_mut() { Ok(rhs) } else { Err(NotAnApp) }
     }
+
+    /// Applies two terms with substitution and variable update (but without reduction), consuming them
+    /// in the process.
+    ///
+    /// # Example
+    /// ```
+    /// use lambda_calculus::term::apply;
+    /// use lambda_calculus::parser::parse;
+    ///
+    /// let lhs    = parse(&"λλ42(λ13)").unwrap();
+    /// let rhs    = parse(&"λ51").unwrap();
+    /// let result = parse(&"λ3(λ61)(λ1(λ71))").unwrap();
+    ///
+    /// assert_eq!(lhs.apply(rhs), Ok(result));
+    /// ```
+    pub fn apply(mut self, rhs: Term) -> Result<Term, Error> {
+        _apply(&mut self, rhs, 0);
+
+        Ok(try!(self.unabs()))
+    }
+
+    /// Reduces an `App`lication by substitution and variable update.
+    ///
+    /// # Example
+    /// ```
+    /// use lambda_calculus::term::{app, abs};
+    /// use lambda_calculus::term::Term::Var;
+    /// use lambda_calculus::arithmetic::zero;
+    /// use lambda_calculus::combinators::i;
+    ///
+    /// assert_eq!(app(i(), zero()).eval(), Ok(abs(abs(Var(1)))));
+    /// ```
+    pub fn eval(self) -> Result<Term, Error> {
+        let (mut lhs, rhs) = try!(self.unapp());
+
+        _apply(&mut lhs, rhs, 0);
+
+        Ok(try!(lhs.unabs()))
+    }
 }
 
 /// Wraps a term in an abstraction. Consumes its argument.
@@ -229,7 +268,8 @@ impl Term {
 /// ```
 pub fn abs(term: Term) -> Term { Abs(Box::new(term)) }
 
-/// Produces an application of its arguments, consuming them in the process.
+/// Produces an application of its arguments without substitution or reduction, consuming them in
+/// the process.
 ///
 /// # Example
 /// ```
@@ -240,17 +280,68 @@ pub fn abs(term: Term) -> Term { Abs(Box::new(term)) }
 /// ```
 pub fn app(lhs: Term, rhs: Term) -> Term { App(Box::new(lhs), Box::new(rhs)) }
 
-fn parenthesize_if(condition: bool, input: &str) -> Cow<str> {
-    if condition {
-        format!("({})", input).into()
-    } else {
-        input.into()
+/// Applies two terms with substitution and variable update (but without reduction), consuming them
+/// in the process.
+///
+/// # Example
+/// ```
+/// use lambda_calculus::term::apply;
+/// use lambda_calculus::parser::parse;
+///
+/// let lhs    = parse(&"λλ42(λ13)").unwrap();
+/// let rhs    = parse(&"λ51").unwrap();
+/// let result = parse(&"λ3(λ61)(λ1(λ71))").unwrap();
+///
+/// assert_eq!(apply(lhs, rhs), Ok(result));
+/// ```
+pub fn apply(mut lhs: Term, rhs: Term) -> Result<Term, Error> {
+    _apply(&mut lhs, rhs, 0);
+
+    Ok(try!(lhs.unabs()))
+}
+
+fn _apply(lhs: &mut Term, rhs: Term, depth: usize) {
+    match *lhs {
+        Var(i) => if i == depth {
+            *lhs = rhs; // substitute a top-level variable from lhs with rhs
+            update_free_variables(lhs, depth - 1) // update free variables' indices from rhs
+        } else if i > depth {
+            *lhs = Var(i - 1) // decrement a free variable's index
+        },
+        Abs(_) => {
+            _apply(lhs.unabs_ref_mut().unwrap(), rhs, depth + 1)
+        },
+        App(_, _) => {
+            _apply(lhs.lhs_ref_mut().unwrap(), rhs.clone(), depth);
+            _apply(lhs.rhs_ref_mut().unwrap(), rhs, depth);
+        }
+    }
+}
+
+fn update_free_variables(term: &mut Term, depth: usize) {
+    match *term {
+        Var(i) => if i > depth {
+            *term = Var(i + depth)
+        },
+        Abs(_) => {
+            update_free_variables(term.unabs_ref_mut().unwrap(), depth)
+        },
+        App(_, _) => {
+            update_free_variables(term.lhs_ref_mut().unwrap(), depth);
+            update_free_variables(term.rhs_ref_mut().unwrap(), depth);
+        }
     }
 }
 
 /// Set to `true` for λ or `false` for \ when displaying lambda terms. The default is `true`.
 pub const DISPLAY_PRETTY: bool = true;
 
+impl fmt::Display for Term {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(f, "{}", show_precedence(0, self))
+    }
+}
+
 fn show_precedence(context_precedence: usize, term: &Term) -> String {
     match *term {
         Var(i) => format!("{:X}", i), // max. index = 15
@@ -265,15 +356,31 @@ fn show_precedence(context_precedence: usize, term: &Term) -> String {
     }
 }
 
-impl fmt::Display for Term {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "{}", show_precedence(0, self))
+fn parenthesize_if(condition: bool, input: &str) -> Cow<str> {
+    if condition {
+        format!("({})", input).into()
+    } else {
+        input.into()
     }
 }
 
 #[cfg(test)]
 mod test {
-    use arithmetic::*;
+    use arithmetic::{zero, succ, pred};
+    use combinators::i;
+    use parser::parse;
+    use term::{apply, abs};
+    use term::Term::Var;
+
+    #[test]
+    fn applying() {
+        let lhs    = parse(&"λλ42(λ13)").unwrap();
+        let rhs    = parse(&"λ51").unwrap();
+        let result = parse(&"λ3(λ61)(λ1(λ71))").unwrap();
+        assert_eq!(apply(lhs, rhs), Ok(result));
+
+        assert_eq!(i().app(zero()).apply().unwrap(), abs(abs(Var(1))));
+    }
 
     #[test]
     fn displaying_terms() {
